#% -*- mode: tm; mode: fold;  -*-
#%{{{Macros 

#i linuxdoc.tm

#d slang S-Lang
#d gtk Gtk
#d slgtk SLgtk
#d slirp SLIRP
#d slsh \tt{slsh}
#d kw#1 \tt{$1}
#d exmp#1 \tt{$1}
#d var#1 \tt{$1}
#d ldots ...
#d chapter#1 <chapt>$1<p>
#d preface <preface>
#d tag#1 <tag>$1</tag>
#d tsb \begin{tscreen}
#d tse \end{tscreen}
#d img#1 <figure><ph vspace="5cm"><img src="$1"></figure>

#d function#1 \sect{<bf>$1</bf>\label{$1}}<descrip>
#d variable#1 \sect{<bf>$1</bf>\label{$1}}<descrip>
#cd function#1 <p><bf>$1</bf>\label{$1}<p><descrip>
#d synopsis#1 <tag> Synopsis </tag> $1
#d keywords#1 <tag> Keywords </tag> $1
#d usage#1 <tag> Usage </tag> <tt>$1</tt>
#d description <tag> Description </tag>
#d example <tag> Example </tag>
#d notes <tag> Notes </tag>
#d seealso#1 <tag> See Also </tag> <tt>$1</tt>
#d r#1 \ref{$1}{$1}
#d done </descrip><p>
#d -1 <tt>-1</tt>
#d 0 <tt>0</tt>
#d 1 <tt>1</tt>
#d 2 <tt>2</tt>
#d 3 <tt>3</tt>
#d 4 <tt>4</tt>
#d 5 <tt>5</tt>
#d 6 <tt>6</tt>
#d 7 <tt>7</tt>
#d 8 <tt>8</tt>
#d 9 <tt>9</tt>
#d NULL <tt>NULL</tt>
#d documentstyle book

#%}}}

#d gtktutorial_url \url{http://www.gtk.org/tutorial/}{GTK tutorial}
#d gtkdocs_url \url{http://www.gtk.org/api/}{online GTK documentation}
#d slirp_url \url{http://space.mit.edu/~mnoble/slirp/}{SLIRP website}

\linuxdoc
\begin{\documentstyle}

\title Reference Manual for \slgtk Version 0.7.6

\author Michael S. Noble, \tt{mnoble@space.mit.edu}
\date \__today__

\toc

#i preface.tm

\chapter{Introduction}

  The \slgtk package binds the GIMP Toolkit, also known as \gtk, to the
  \slang scripting language.  It provides an importable module which 
  makes most of \gtk and its constituent libraries callable directly
  from \slang scripts.  With \slgtk the \slang programmer now has access
  to a powerful, cross-platform widget set for creating sophisticated
  graphical user interfaces (GUIs).  Bindings to a subset of the GtkExtra
  widget set are also provided, for 2D visualization.

  A driving force behind the development of \slgtk has been the notion
  of the \em{guilet} (pronounced "gooey-let"), by which we mean to connote
  visual interfaces of a small-ish, scriptable nature, that may be easily
  embedded within -- and launched from -- other programs, even applications
  with interactive command lines and no ostensible graphical interface.

  As such guilets fill an important software niche, the middle ground
  between novice users -- who often argue that visual interfaces make
  software easier to use -- and power users -- who frequently complain
  that GUIs are more cumbersome and inflexible than command lines.
  Experience suggests that the code size and development cycle of the typical
  guilet is considerably smaller than that of traditional, monolithic GUIs
  (by which we mean to connote applications coded from scratch to have 
  only a visual interface, typically utilizing a compiled language in
  conjunction with lower-level toolkits such as Xt, Motif, or the 
  Microsoft's Windows Foundation Classes).  Moreover, given their scripting
  heritage, guilets can be more amenable to changing requirements or ad-hoc
  experimentation and customization by the end-user.

  Other benefits of embedded guilets are the memory conservation and 
  seamlessness of operation which can result from having visualization 
  capabilities \em{located more closely to the data upon which they 
  will operate.}  That is, it can be vastly preferable to generate
  sophisticated graphics from within the same address space of the
  process holding the data to, say, model plasma emission, rather than
  having to invoke a distinct process (loading the entire dataset again)
  to generate, say, a 3D image or surface plot from such.  It is 
  increasingly common, within astronomy for
  instance, for scientists to interactively analyze datasets 500 megabytes
  and greater in size.  As dataset sizes scale even higher even the most
  powerful desktop workstation will starve for memory if several copies
  of such behemoths must be simultaneously resident in RAM.

\sect{Code Generation}

  Most of the over 2000 functions present in the \slgtk module are
  created automatically by inspection of the header files of Gtk 
  and its constituent libraries, using the bundled \em{\slirp} code
  generator (documentation available online at the \slirp_url).  Note
  that a list of functions omitted
  by the code generator is created during the \slgtk build process, and
  that the \kw{checkgtk} script may also be used to determine whether
  or a given function has been wrapped.

\sect{Non-interactive Execution}
   Given their role as graphical toolkits \gtk and \slgtk will most
   frequently be used interactively, e.g. from an X windows session.
   Sometimes, however, one might like to utilize them within a
   windowless environment, such as a cron job or remotely-executed
   regression test.  Towards that end \slgtk bundles the xvfb-run
   script, which is described in chapter \r{Utilities}.
   
\chapter{Some Examples}

  At this point you would probably like to see an example of the
  \em{Hello World} variety, so here is our very own \tt{howdy.sl}:
#v+
#i howdy.sl
#v-
   This script may be executed in any application which embeds an \slang 
   interpreter endowed with the capacity to \kw{import()} modules.  The
   \slsh shell is one such program, which if invoked on Unix/Linux
   (assuming you've installed \slgtk) as follows
#v+
  unix%  slsh ./howdy.sl
#v-
   should raise on your display an image which looks something like:
#v+

#v-

\img{howdy.png}

#v+
#v-
  The next example shows how to use the powerful Gtk font selector, and 
  is taken directly from the \slgtk \kw{examples/fontsel.sl} sample code.
#v+
#i ../../examples/fontsel.sl

#v-
\img{fontsel.png}

  Note that a wealth of sample guilets are packaged within the \slgtk
  distribution.  The \tt{examples} directory alone contains more than
  than 4000 lines of code, in roughly 40 working guilets (largely
  adaptations from testgtk.c packaged with Gtk), while the \tt{packages}
  directory contains the vwhere() guilet and supporting scripts.

\chapter{Programming Interface}

  The \slgtk application programming interface (api) aims at providing
  a faithful mirror of the apis of the \gtk widget set and selected 
  portions of its dependencies (e.g Gdk, GObject, GLib, and Pango).  As
  such we do
  not attempt to exhaustively document here every function wrapped from 
  these libraries, but rather will document the discrepancies between 
  the two apis and refer the reader to the \gtkdocs_url.

  Knowledgeable Gtk programmers should find the mapping from Gtk to
  \slgtk rather straightforward, and should be pleased by the absence
  of casting and memory management concerns, and the shorter, simpler
  code that results.  Developers with no prior Gtk experience are 
  encouraged to peruse the \gtktutorial_url, as well as the \slgtk
  examples and packages directories mentioned above.
  
  The remaining sections describe the major discrepancies between the \slgtk
  and Gtk interfaces.  The CHANGELOG file within the distribution also 
  provides valuable supplemental information.

\sect{Memory Management}

  Two of the most significant benefits of coding Gtk guilets in \slang are
  that the user may remain largely ignorant of explicit memory management
  and type casting concerns.  In contrast with Gtk C programs, which
  expose substantial use of casting macros, casting is never required
  in SLgtk guilets.  There should likewise be virtually zero need to
  \em{explicitly} unref, sink, or destroy Gtk, Gdk, or GLib objects 
  instantiated in \slang scope, as \slgtk will automatically call the 
  appropriate finalizer when the \slang variable goes out of scope. 
  In other words, it is not necessary to

  \begin{itemize}
    \item \tt{gdk_cursor_destroy() a GdkCursor}
    \item \tt{gdk_gc_destroy() a graphics context (GdkGc)}
    \item \tt{gdk_region_destroy() a GdkRegion}
    \item \tt{gtk_object_sink(), g_object_ref(), g_object_unref() a GtkTooltips}
    \item \tt{g_object_unref() bitmaps, pixbufs, or pixmaps (GdkBitmap, GdkPixbuf, GdkPixmap)}
    \item \tt{g_object_get_data()/g_object_unref() after g_object_set_data()}
    \item \tt{gtk_text_iter_free() a GtkTextView iterator (GtkTextIter)}
  \end{itemize}

  The obvious exception to this is that if your \slang code explicitly adds
  a reference to an instance then it should likewise explicitly remove the
  reference later.

\sect{Gtk \#define Macros}

  Several \tt{\#define} macros for querying or manipulating the state of
  GtkWidget instances have been wrapped for use in \slang scope.  Currently
  these, and their corresponding \slang name, are:
#v+
 	GTK_WIDGET_VISIBLE(widget)		gtk_widget_visible(widget)
	GTK_WIDGET_MAPPED(widget)		gtk_widget_mapped(widget)
	GTK_WIDGET_REALIZED(widget)		gtk_widget_realized(widget)
	GTK_WIDGET_TOPLEVEL(widget)		gtk_widget_toplevel(widget)
	GTK_WIDGET_STATE(widget)		gtk_widget_state(widget)
	GTK_WIDGET_IS_SENSITIVE(widget)		gtk_widget_is_sensitive(widget)
	GTK_WIDGET_SET_FLAGS(widget,flag)	gtk_widget_set_flags(widget,flag)
	GTK_WIDGET_UNSET_FLAGS(widget,flag)	gtk_widget_unset_flags(widget,flag)
#v-

\sect{Variable Length Argument Lists}

  Because the \slang interpreter records how many arguments have been passed
  to a function, those \slgtk functions which wrap C functions evincing a
  variable number of arguments generally do not require any special 
  termination parameter.  For example, it is not necessary to:
  \begin{itemize}
    \item terminate calls to \tt{gtk_list_store_set()} with -1
    \item terminate calls to \tt{gtk_tree_view_column_new_with_attributes()}
    with NULL
    \item terminate calls to \tt{gtk_text_buffer_create_tag()} with NULL
  \end{itemize}

\sect{Widget Field Accessor Functions}
  In most cases it is good that \slang code is not permitted to peer within
  Gtk structure internals at runtime, since it enforces the encapsulation
  provided by the object abstraction.  For those instances, though, where
  Gtk itself does not enforce object encapsulation (by allowing widget 
  internals to be exposed in user code through direct structure field 
  references, rather than object accessor functions), \slgtk provides a
  corresponding _get() wrapper function to access the field.

  One example of such a function is \tt{gtk_dialog_get_vbox()}. Another is
  \tt{gtk_color_selection_dialog_get_ok_button()}, while the complete
  list can be found in the file \tt{src/faccessors_ftable.c} generated
  during the \slgtk build.

\sect{Typing}

  As discussed below all Gtk object, widget, and structured types map 
  to either \em{structure} or \em{opaque} \slang types.  The latter
  should be viewed as handles to internal data of an unspecified type,
  created by the S-Lang memory managed type (MMT) mechanism.  The reader
  is encouraged to read the documentation, available at the \slirp_url,
  to better understand how types are mapped between \slang and \gtk.
  
\sect1{Unique \slang Opaque Types}

  At import time \slgtk introduces the following unique \slang types:
#v+
#i opaques-base.txt
#v-
  Upcasts and downcasts may be safely performed between a parent type and
  any of its ancestors, and \slgtk will automatically cast a given opaque
  instance variable to the type most appropriate for a given call.  Attempting
  a cast between siblings, or other incompatible types, will signal an error.

  Note that the function \tt{gtk_object_type()}, a wrapper for the 
  GTK_OBJECT_TYPE() C macro, may be used to query the underlying Gtk 
  type of the C variable encapsulated by an opaque \slang variable.
  Likewise, \tt{gtk_object_type_name()}, which wraps the
  GTK_OBJECT_TYPE_NAME() C macro, may be used to query the name of
  underlying Gtk type.
#v+



#v-

\sect1{Gtk to \slang Type Mappings}

  These new opaque types map most of the GObject and Gtk class hierarchy to
  \slang as follows, where \slang types are prefixed with a plus and
  C types are suffixed with an asterisk:
#v+
#i opaques-all.txt
#v-
\sect1{GTypes and Properties}
  
  Most of dynamic typing system provided by GLib is not wrapped, so users 
  cannot presently register S-Lang-scoped types as GTypes.  There should 
  be very little call for this in \slang scripts, but should the need 
  arise the developer would be better off defining such types in C scope 
  and upwardly promoting the registered GType instances to \slang scope,
  rather than trying to map a pure \slang type downward to a C-scoped GType.

  The batch widget construction and property setting mechanisms, which 
  accept variable length, variable-typed argument lists (e.g.,
  \tt{gtk_widget_new()} or \tt{g_object_set()}), are likewise unsupported.

\sect1{GdkColor}

  The \slang version of \tt{gdk_color_parse()} is cleaner, simpler to use, and
  more powerful than its C counterpart.  Color parsing and allocation
  are achieved in one step, with no variable references needed.  Compare
#v+
  	variable red = gdk_color_parse("red");
#v-
  in \slang with the comparable code in C
#v+
  	GdkColor red;
	gdk_color_parse("red", &color);
	gdk_color_alloc(gdk_colormap_get_system(), &color);
#v-
  Also, since it returns a GdkColor structure instead of a gboolean, the
  \slgtk wrapper for gdk_color_parse() will return NULL when it cannot 
  either parse or allocate the color requested.
  
  The \tt{gtk_color_selection_get_current_color()} routine has been
  likewise modified to take only 1 argument and return its GdkColor
  result on the stack (or NULL on failure).

  The \tt{gdk_color_copy()} and \tt{gdk_color_free()} routines are
  intentionally not wrapped.  The need for the latter is questionable,
  while the
  functionality of the former can be achieved with the \slang @ operator:
#v+
  	variable red = gdk_color_parse("red");
	variable red_copy = @red;
#v-
  Other \slgtk wrappers of C routines which accept a GdkColor* likewise do 
  not require that a GdkColor reference be passed.  For example, consider 
  the C implementation of \tt{gtk_widget_modify_fg()}, called idiomatically
  as
#v+
   	GdkColor color;
	...
	gdk_color_parse("red",&color);
	...
	gtk_widget_modify_fg(...,&color);
#v-
  The matching \slgtk idiom is
#v+
   	variable color = gdk_color_parse("red");
	...
	gtk_widget_modify_fg(...,color);
#v-

\sect1{GdkRectangle}

  GdkRectangle structures may be instantiated in \slang scope by either
  calling the function
#v+
	rect1 = gdk_rectangle_new(x,y,width,height);
#v-
  or using the \slang dereference operator on an existing GdkRectangle
  instance:
#v+
	rect2 = @rect1;
#v-
  Note that \tt{gdk_rectangle_new()} does not exist in Gdk proper, but
  rather is provided by \slgtk as a convenience.

\sect1{GtkAllocation}

  The GtkAllocation structure is simply a GdkRectangle typedef.  Thus the
  GtkAllocation type is mirrored in \slang scope as a proper structure.

\sect1{GdkPoint}

  Likewise, GdkPoint structures may be instantiated in \slang scope either
  by calling the convenience function 
#v+
	p1 = gdk_point_new(x,y);
#v-
  or using the \slang dereference operator on an existing GdkPoint instance:
#v+
	p2 = @p1;
#v-

\sect1{GdkEvent}

  Each \slang GdkEvent structure will reflect the three core GdkEventAny
  fields: \tt{type}, \tt{window}, and \tt{send_event}.  Additionally,
  motion notify and button events contain integral \tt{x}
  and \tt{y} coordinate fields, while keypress events also contain a 
  \tt{keyval} field, and expose events will reflect the \tt{area} field.

\sect1{GtkTextIter}

  The family of \tt{GtkTextBuffer} routines which return a \tt{GtkTextIter}
  do so by putting the result on the stack, instead of using a reference 
  parameter as in the C api.  This fosters natural S-Lang usages, such as
#v+
	variable iter = gtk_text_buffer_get_iter_at_offset(buffer,0);
#v-
  which are cleaner and simpler than their C equivalents:
#v+
	GtkTextIter iter;
	gtk_text_buffer_get_iter_at_offset(buffer,&iter,0);
#v-
\sect1{GError}

  Unlike in C, it is not necessary to explicitly NULL out GError variables
  intended to receive a GError reference upon return from a function 
  invocation.  Such variables are guaranteed to have a legal \slang 
  value (possibly NULL) upon return.

\sect1{GtkMenu}

Note that the Gtk menu popup function has been simplified in SLgtk from
#v+
	gtk_menu_popup(menu, parent_menu_shell, parent_menu_item,
			menu_position_func, func_data, mouse_button_num,
			activate_time);
#v-
to
#v+
	gtk_menu_popup(menu, mouse_button_num, activate_time);
#v-
Per the Gtk documentation, the parent menu and position function parameters
are typically unused.

\sect1{Screen Depths}
\tt{gdk_query_depths(int **depths, int *num)} has been wrapped so that it
may be called from \slang scope with zero arguments, with the depths array
returned on the stack:
#v+
	Integer_Type[]	gdk_query_depths()
#v-

\sect{Signal Handlers and Callbacks}

  Gtk and GObject functions which install timeout or signal handlers --
  such as \tt{g_signal_connect()}, \tt{gtk_timeout_add()}, or 
  \tt{gtk_idle_add()} -- return a positive integer uniquely identifying
  the registered callback or, though this does not appear to be stated 
  explicitly in the Gtk reference, zero upon error.

  The core signal connection mechanisms, \tt{g_signal_connect()} and 
  \tt{g_signal_connect_swapped()} are considerably more flexible than
  their C counterparts, in that both transparently construct GObject 
  closures when called.  When registering signal handlers the caller
  may thus specify as many \em{callback data} parameters as desired,
  including zero.  The same holds true for callback functions registered
  via \tt{gtk_idle_add}, \tt{gtk_quit_add}, \tt{gtk_timeout_add}, and
  \tt{gtk_container_foreach}.

\sect{Callback Errors}

  If an \slang callback function -- or other \slang function(s) called from
  within an \slang callback -- signals a non-recoverable error (as indicated
  by the C-scoped SLang_Error variable having a value less than zero), then
  the top level window will be destroyed and the outermost main loop will
  be terminated via \tt{gtk_main_quit()}.  In the typical case, since many
  scripted GUIs will not nest \tt{gtk_main()} loops, this will cause the
  GUI to terminate entirely.

  If such functions signal a recoverable error, then the SLang interpreter
  will be restarted and execution resumed normally, possibly with detritus
  left on the stack.

\chapter{VWhere: N-Dimensional Data Mining}

#i vwhere.tm

#%\sect{_gtk_main_quit_when_idle()}
#%
#%  This function is similar to the \gtk function \tt{gtk_main_quit()}, 
#% except that it provides a guarantee that the innermost main loop (if
#%  one is running) will eventually be terminated.

\chapter{Image Display}

#i imdisplayfun.tm

\chapter{Printing}

\sect{_print_dialog}
#v+
        Void_Type        _print_dialog(print_context, callback [, cbarg1, ...])
#v-
  This function will create a modal dialog which your guilet can use to
  query the user for common printing options.  The print context argument 
  can either be NULL (in which case a default context will be generated) or
  an instance of the \tt{GtkPrintContext} structure, with fields
#v+
   title    text to display in dialog window title bar (default: "Print")
   dest     destination of generated output: 0 = printer (default), 1 = file
   cmd      operating system command used to generate a print job (default: "lpr")
   file     an output file name (default: "slgtk.ps")
   orient   page orientation: 0 = portrait (default), 1 = landscape
   size     page size: 0 = letter (default), 1 = legal, 2 = A4
#v-

  The second argument is a reference to a function that will be called
  when the user presses the \tt{Print} dialog button.   This function
  should be defined to receive at least one argument (the print context,
  updated to reflect any input given within the dialog), and optionally
  any callback arguments specified when the dialog was invoked.  It is
  is important to understand that this callback, rather than the print
  dialog itself, is responsible for output generation.

  The following \slang snippet gives a mock usage of the print dialog,
  and intentionally avoids initializing several print context fields
  in order to show that the dialog gives them suitable default values.
#v+
define print_cb(context, callback_message)
{

   vmessage(callback_message);			% sample callback arg usage

   variable file;
   if (context.dest == 0)			% generate output to printer
	file = "temp.ps";			% mock temp file name
   else
	file = context.file;			% else dump output to user-
	   					% specified filename

   % the print dialog uses a callback function to provide the flexibility
   % of calling an application-specific output rendering method, such as
   % the following mocked-up "export_postscript" function

   %() = export_postscript(file);

   if (context.dest == 0) {
	() = system( sprintf("%s %s",context.cmd, file));
	() = remove(file);
   }
}

define launch_print_dialog()
{
   variable context = @GtkPrintContext;
   context.title = "MyApp Print Dialog";
   context.file = "MyApp.ps";
   _print_dialog(context, &print_cb, "Hello, from MyApp print callback!");
}
#v-

\sect{_print_context_new()}
#v+
        GtkPrintContext =  _print_context_new()
#v-
  This function will generate a \kw{GtkPrintContext} structure with
  default values for all fields.

\chapter{Plotting}

  \slgtk also wraps much of the functionality of the

  \begin{itemize}
    \item \tt{GtkPlot}
    \item \tt{GtkPlotCanvas}
    \item \tt{GtkPlotData}
  \end{itemize}
  
  widgets from the GtkExtra widget set, with the aim of providing a
  lightweight mechanism for 2D plotting and data exploration, with support
  for PostScript hardcopy.  While these functions may be used interactively,
  as is, they are probably more suitable as building blocks for higher-level
  plotting functions.  Towards that end \slgtk introduces the
  \tt{GtkPlotDescriptor} \slang type, a structured container, which can
  be created and manipulated by the functions described below.  Examples
  of their use may be gleaned from the \tt{vwhere} guilet described above.

\sect{_gtk_plot}
#v+
        GtkPlotDescriptor = _gtk_plot(x, y [,color [, style]])
#v-
  This function will create a plot canvas, draw upon it an 2D scatterplot
  from the given vectors (optionally in the given line color and style), and
  return
  an \slang structure containing both.  To visualize the result, add the
  \tt{canvas} field of the returned structure to a suitable container.

  A custom canvas size may be specified via the \tt{width=} and \tt{height=}
  qualifiers, in units of pixels.

\sect{_gtk_oplot}
#v+
       _gtk_oplot(GtkPlotDescriptor, x, y, [, color [, style]])
#v-

  This function will overplot the given vectors (optionally in the given
  line color and style) upon the canvas contained within the given plot
  descriptor.  A custom canvas size may be specified via the \tt{width=}
  and \tt{height=} qualifiers, in units of pixels.

\sect{_gtk_plot_set_x_range, _gtk_plot_set_y_range}

#v+
	_gtk_plot_set_xrange(GtkPlotDescriptor, xmin, xmax)
	_gtk_plot_set_yrange(GtkPlotDescriptor, ymin, ymax)
#v-

  These functions customize the tick marks of the bounding box in which
  the plots are displayed.

\sect{_gtk_plot_redraw}
#v+
	_gtk_plot_redraw(GtkPlotDescriptor)
#v-
  This is a convenience function which repaints the internal canvas pixmap
  and then refreshes the current display with the new pixmap.  Note that
  plots containing many points may require several seconds to perform this
  combined action.  If multiple canvas operations are to be performed --
  each of which logically induce a repaint -- a better choice may be to avoid
  this function altogether, and instead freeze the canvas, perform the 
  necessary operations, thaw the canvas, and then paint the canvas or refresh
  the display explicitly.

\sect{_gtk_plot_remove}
#v+
	_gtk_plot_remove(GtkPlotDescriptor, i)
#v-
  Eliminate the \em{i}th plot from the given descriptor, causing it to
  be erased from the display upon the next redraw.  The index \em{i}
  may take the values 1 through N, where N is the number of plots that
  have been drawn to the descriptor.

\chapter{Miscellaneous}

  \slgtk provides an assortment of other higher-level \slang functions and
  variables, to simplify common case use of selected Gtk widgets or features.
  The names of some of these are prefixed with underscores, usually to
  distinguish them from wrappers for \gtk functions proper or denote
  their higher susceptibility to revision in a future release.

\sect{GdkColor Variables}

  As a convenience \slgtk pre-allocates a number of GdkColor structures,
  for colors expected to be commonly used.  They are named
  \begin{itemize}
    \item \tt{gdk_red}
    \item \tt{gdk_green}
    \item \tt{gdk_blue}
    \item \tt{gdk_black}
    \item \tt{gdk_grey}
    \item \tt{gdk_white}
  \end{itemize}
  and will be available in the relevant namespace after \tt{gtk.sl} has
  been loaded.
  
\sect{_menu_new}
#v+
        GtkWidget = _menu_new(String_Type labels[], callback [,cbdata1, cbdata2, ...] )
#v-
  A convenience function to instantiate a GtkMenu, whose entries are labels
  created from the given array of strings.  The callback specification may 
  either be \kw{NULL} or a signature (a function reference, optionally
  followed by one or more callback data parameters) suitable for responding
  to the \em{activate} signal via \tt{g_signal_connect()}.
  Recall that such callback functions should accept at least one argument,
  even when no callback data is registered, to receive the activated menu
  item.  Any label which, after being lowercased, matches "<separator>"
  will cause a horizontal separator to be drawn in the menu.

\sect{_option_menu_new}
#v+
        GtkWidget = _option_menu_new(String_Type labels[], Integer_type default)
#v-
  A convenience function to instantiate a GtkOptionMenu, whose entries
  are labels created from the given array of strings.  The active menu
  item will be set to the given default.

\sect{_color_button_new}
#v+
        GtkWidget = _color_button_new(GdkColor_structure)
#v-
  This function returns an instantiated GtkButton, which when realized
  will display a swatch of the given color.  When the button is pressed
  a color selection dialog will be launched through which the color of
  the swatch (as well as the RGB field values within the given GdkColor
  structure, since \slang structures are passed as references) may be
  modified.

\sect{_info_window}
#v+
        Void_Type        _info_window(title,text)
#v-
  This function will create a non-modal dialog to display the given text
  in a window with the given title.  The dialog will be raised immediately,
  provided there is at least one active \tt{gtk_main()} loop context.

\sect{_is_numeric}
#v+
        Integer_Type  =  _is_numeric(Any_Type datum)
#v-
  This function will return a one if the given datum can be interpreted
  as a real- or integral-valued numerical type, and zero otherwise.  This
  function will be phased out of \slgtk proper, as it is now available in
  \slang 2.x.

\sect{_is_callable}
#v+
        Integer_Type  =  _is_callable(Any_Type datum)
#v-
  This function returns either 1 or 0, to indicate whether the given
  datum can be dereferenced to invoke a function.  This function will be
  phased out of \slgtk proper, as it is now available in \slang 2.x.

\sect{_get_slgtk_doc_string}
#v+
        String_Type = _get_slgtk_doc_string(topic)
#v-
  This function will search the \slgtk documentation for the given topic.
  If the topic is found then its entire entry will be returned as a single
  string, otherwise an error string will be returned.

\sect{_gtk_window_destroy_with(window,parent)}

  Provides a one-step way to set the transient parent of a window AND have
  it be destroyed with that parent.  It is equivalent to calling
#v+
	gtk_window_set_transient_for(window,parent);
	gtk_window_destroy_with_parent(window,TRUE);
#v-

\sect{_gdk_pixbuf_get_formats()}
#v+
        Array_Type = _gdk_pixbuf_get_formats()
#v-
  Similar to the Gdk function \tt{gdk_pixbuf_get_formats()}, in that
  its return value indicates the number and kind of file formats
  supported by the Gdk pixbuf interface, only this version returns an
  array of \tt{String_Type} instead of a \tt{GSList*}.

\chapter{Utilities}
 \label{Utilities}

\sect{slgtksh}
  Included within \tt{src/Makefile} of the \slgtk distribution is a target
  which builds \em{slgtksh}, a version of the \slang shell, \slsh, with
  \slgtk statically linked.  See the \slirp \tt{importify} utility for details.

\sect{imdisplay}

  This is a wrapper around the imdisplay() function described above,
  suitable for use from the operating system command line.  Input
  images are specified by file name, and all of the same transform
  options are supported.
#v+
  	unix% imdisplay
	Usage: imdisplay file_or_option [ file_or_option ...]
#v-

\sect{importify}
  This \slirp utility script provides a simple mechanism for building the
  \slang shell, \slsh with one or more importable modules statically
  linked.
#v+
  	unix% <slgtk>/slirp/importify -h

	importify, version 0.9 (mnoble@space.mit.edu)
	Emits to stdout a copy of slsh.c, modified to facilitate static
	loading of one or more embedded S-Lang modules.

	Usage: importify [options] module_name [ module_name ...]

	Options:
	-h                   generate this message
	-m symbol            allow main() to be overridden, by renaming it to symbol()
	-p                   do not generate prototype for module init func
	-q                   generate quiet code (no startup messages)
	-s path_to_slsh.c    useful when slsh.c is not in $PWD
#v-
  Using modules in this manner simplifies debugging in two ways.  First,
  since the module entry points are known at invocation, breakpoints can
  be set in module code, or within the library wrapped by the module,
  \em{prior} to launching the process.  In contrast, when modules are
  imported into an \slang -enabled application at runtime (i.e., after
  launching the process) its entry points are made available to the 
  debugger \em{only} after the module has been fully loaded (on UNIX
  systems, for example, after a successful dlopen() call), which makes
  setting breakpoints in the module code (or its wrapped library) much
  more difficult.
  
  A second benefit of using a statically loaded module is that breakpoints
  set within the module persist when the parent application is restarted
  from within the debugger.  With code imported at runtime such breakpoints
  are lost each time the parent application is restarted.

\sect{slirp_debug_pause}
  In situations where it is not desirable or possible to use slgtksh to 
  debug your guilet, you may instead use the \tt{slirp_debug_pause()}
  stub routine.

  To activate this stub set the SLIRP_DEBUG_PAUSE environment variable before
  importing the \slgtk module.  This will cause the parent process to
  wait a specified amount of time prior to entering the \gtk main loop.
  During this time you may set breakpoints within the codebase of the
  \slgtk module (since its symbol table will have been loaded prior to
  the invocation of the debugging stub), including Gtk and its constituent
  libraries, or within your own compiled code.

  If SLIRP_DEBUG_PAUSE is unset in the environment the debugging stub will
  do nothing.  Otherwise, if the variable evaluates to a negative
  integer \em{N} \tt{slirp_debug_pause()} will sleep for abs(N) seconds.
  If SLIRP_DEBUG_PAUSE is set to any other value the stub will pause 
  indefinitely, awaiting a keypress in the window from which the parent
  process was launched.
  
  More information is available on the \slirp_url.

\sect{slgtk-demo}
  This script provides a convenient way to execute the demonstration code
  in the examples directory.  The usage synopsis is:
#v+
	slgtk-demo [--auto | guilet_name ]
#v-
  The --auto option will cause the script to cycle through raising and
  lowering the entire suite of example guilets, and serves as a useful
  way to exercise the module at large.  When invoked with the name of a
  specific guilet the script will launch the parent guilet and additionally
  raise the named example.  When invoked with no options only the parent
  guilet will be raised.

\sect{checkgtk}
   This script allows the developer to quickly identify whether
   or not a given Gtk/Gdk/Pango/etc C function is available within
   the \slgtk module.
#v+
	unix% <slgtk>/src/checkgtk
	Usage: checkgtk gtk_function_name [ gtk_function_name ...]
#v-
\sect{xvfb-run}
   The \tt{xvfb-run} script supports the use of \slgtk in a display-less
   environment, such as a cron job or remotely-executed regression test.
   Intended for Unix-compatible systems running X11, the script was taken
   from Debian Linux and modified slightly to enhance its portability to
   non-Linux systems.  It, along with the \tt{mcookie} utility and
   \tt{md5} algorithm
   from the \tt{util-linux} package, may be found within the \tt{src/batch}
   directory of the distribution.

\chapter{Versioning}

  \slgtk indicates version information in the variables:
#%#v+
#%	_slgtk_major_version
#%	_slgtk_minor_version
#%	_slgtk_micro_version
#%#v-
#%  To conform with the nascent \slang modules development standard \slgtk
#% also provides:
#v+
   _gtk_module_version		An integer containing the value
				(10000*major_ver) + (100*minor_ver) + micro_ver

   _gtk_module_version_string	A string containing the value
				major_ver.minor_ver.micro_ver
#v-
   As a user convenience \slgtk also indicates the version of Gtk
   against which it was built:
#v+
   _gtk_major_version
   _gtk_micro_version
   _gtk_minor_version
#v-

\end{\documentstyle}
